"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Editor = void 0;
const CanvasTools_Filter_1 = require("./CanvasTools.Filter");
const Rect_1 = require("./Core/Rect");
const ISelectorSettings_1 = require("./Interface/ISelectorSettings");
const RegionsManager_1 = require("./Region/RegionsManager");
const ZoomManager_1 = require("./Core/ZoomManager");
const AreaSelector_1 = require("./Selection/AreaSelector");
const ToolbarIcon_1 = require("./Toolbar/ToolbarIcon");
const Toolbar_1 = require("./Toolbar/Toolbar");
const Point2D_1 = require("./Core/Point2D");
class Editor {
    constructor(container, areaSelector, regionsManager, filterPipeline, isZoomEnabled, readonly) {
        this.autoResize = true;
        this.isRMFrozen = false;
        this.read_only = false;
        this.isondraw = false;
        this.isonmove = false;
        this.read_only = readonly;
        this.contentCanvas = this.createCanvasElement();
        this.editorSVG = this.createSVGElement();
        this.editorContainerDiv = container;
        this.editorContainerDiv.classList.add("CanvasToolsContainer");
        this.editorContainerDiv.tabIndex = 0;
        this.editorDiv = this.createDivElement();
        this.editorDiv.classList.add("CanvasToolsEditor");
        this.editorDiv.append(this.contentCanvas);
        this.editorDiv.append(this.editorSVG);
        this.editorContainerDiv.append(this.editorDiv);
        const rmCallbacks = {
            onChange: null,
            onManipulationBegin: (region) => {
                this.areaSelector.hide();
                if (typeof this.onManipulationBegin === "function") {
                    this.onManipulationBegin(region);
                }
            },
            onManipulationEnd: (region) => {
                this.areaSelector.show();
                if (typeof this.onManipulationEnd === "function") {
                    this.onManipulationEnd(region);
                }
            },
            onRegionSelected: (id, multiselection, regionData) => {
                if (typeof this.onRegionSelected === "function") {
                    this.onRegionSelected(id, multiselection, regionData);
                }
            },
            onRegionMove: (id, regionData) => {
                if (typeof this.onRegionMove === "function") {
                    this.onRegionMove(id, regionData);
                }
            },
            onRegionMoveBegin: (id, regionData) => {
                if (typeof this.onRegionMoveBegin === "function") {
                    this.onRegionMoveBegin(id, regionData);
                }
            },
            onRegionMoveEnd: (id, regionData) => {
                if (typeof this.onRegionMoveEnd === "function") {
                    this.onRegionMoveEnd(id, regionData);
                }
            },
            onRegionDelete: (id, regionData) => {
                if (typeof this.onRegionDelete === "function") {
                    this.onRegionDelete(id, regionData);
                }
            },
        };
        if (regionsManager !== null && regionsManager !== undefined) {
            this.regionsManager = regionsManager;
            regionsManager.callbacks = rmCallbacks;
        }
        else {
            this.regionsManager = new RegionsManager_1.RegionsManager(this.editorSVG, rmCallbacks);
        }
        const asCallbacks = {
            onSelectionBegin: () => {
                this.isRMFrozen = this.regionsManager.isFrozen;
                this.regionsManager.freeze();
                if (!this.isonmove) {
                    this.isondraw = true;
                }
                if (typeof this.onSelectionBegin === "function") {
                    this.onSelectionBegin();
                }
            },
            onSelectionEnd: (regionData) => {
                if (!this.isRMFrozen && !this.read_only) {
                    this.regionsManager.unfreeze();
                }
                if (!this.isonmove) {
                    this.isondraw = false;
                }
                if (typeof this.onSelectionEnd === "function") {
                    this.onSelectionEnd(regionData);
                }
            },
        };
        if (areaSelector !== null && areaSelector !== undefined) {
            this.areaSelector = areaSelector;
            this.areaSelector.callbacks = asCallbacks;
        }
        else {
            this.areaSelector = new AreaSelector_1.AreaSelector(this.editorSVG, asCallbacks);
        }
        if (filterPipeline !== undefined && filterPipeline !== null) {
            this.filterPipeline = filterPipeline;
        }
        else {
            this.filterPipeline = new CanvasTools_Filter_1.FilterPipeline();
        }
        const initZoomCallbacks = {
            onZoomingOut: () => {
                this.onZoom(ZoomManager_1.ZoomDirection.Out);
            },
            onZoomingIn: () => {
                this.onZoom(ZoomManager_1.ZoomDirection.In);
            }
        };
        this.zoomManager = ZoomManager_1.ZoomManager.getInstance(false, initZoomCallbacks);
        this.zoomManager.deleteInstance();
        this.zoomManager = ZoomManager_1.ZoomManager.getInstance(false, initZoomCallbacks);
        if (isZoomEnabled) {
            this.zoomManager.isZoomEnabled = true;
        }
        this.resize(container.offsetWidth, container.offsetHeight);
        this.mergedAPI = new Proxy(this, {
            get: (target, prop) => {
                let p;
                let t;
                if (prop in target) {
                    t = target;
                    p = t[prop];
                }
                else if (prop in target.regionsManager) {
                    t = target.RM;
                    p = t[prop];
                }
                else if (prop in target.areaSelector) {
                    t = target.AS;
                    p = t[prop];
                }
                else if (prop in target.filterPipeline) {
                    t = target.FP;
                    p = t[prop];
                }
                else if (prop in target.zoomManager) {
                    t = target.FP;
                    p = t[prop];
                }
                else {
                    p = undefined;
                }
                if (typeof p === "function") {
                    return (...args) => {
                        p.apply(t, args);
                    };
                }
                else {
                    return p;
                }
            },
        });
        this.subscribeToEvents();
    }
    get api() {
        return this.mergedAPI;
    }
    addToolbar(container, toolbarSet, iconsPath) {
        const svg = this.createSVGElement();
        container.append(svg);
        this.toolbar = new Toolbar_1.Toolbar(svg);
        if (toolbarSet === null || toolbarSet === undefined) {
            toolbarSet = Editor.FullToolbarSet;
        }
        if (this.zoomManager.isZoomEnabled && toolbarSet === Editor.FullToolbarSet) {
            toolbarSet = toolbarSet.concat(Editor.SeparatorIconGroupToolbar).concat(Editor.ZoomIconGroupToolbar);
        }
        let activeSelector;
        toolbarSet.forEach((item) => {
            if (item.type === ToolbarIcon_1.ToolbarItemType.SEPARATOR) {
                this.toolbar.addSeparator();
            }
            else {
                const toolbarItem = {
                    action: item.action,
                    iconUrl: iconsPath + item.iconFile,
                    tooltip: item.tooltip,
                    key: item.key,
                    width: item.width,
                    height: item.height,
                };
                const actionFn = (action) => {
                    item.actionCallback(action, this.regionsManager, this.areaSelector, this.zoomManager);
                };
                if (item.type === ToolbarIcon_1.ToolbarItemType.SELECTOR) {
                    this.toolbar.addSelector(toolbarItem, actionFn);
                    if (item.activate) {
                        activeSelector = item.action;
                        item.actionCallback(activeSelector, this.regionsManager, this.areaSelector, this.zoomManager);
                    }
                }
                else if (item.type === ToolbarIcon_1.ToolbarItemType.SWITCH && !this.read_only) {
                    this.toolbar.addSwitch(toolbarItem, actionFn);
                    this.toolbar.setSwitch(item.action, item.activate && !this.read_only);
                }
                else if (item.type === ToolbarIcon_1.ToolbarItemType.TRIGGER) {
                    this.toolbar.addTrigger(toolbarItem, actionFn);
                }
            }
        });
        this.toolbar.select(activeSelector);
    }
    setNoSelect() {
        this.toolbar.select("none-select");
    }
    async addContentSource(source) {
        const buffCnvs = document.createElement("canvas");
        const context = buffCnvs.getContext("2d");
        if (source instanceof HTMLImageElement || source instanceof HTMLCanvasElement) {
            this.sourceWidth = source.width;
            this.sourceHeight = source.height;
        }
        else if (source instanceof HTMLVideoElement) {
            this.sourceWidth = source.videoWidth;
            this.sourceHeight = source.videoHeight;
        }
        buffCnvs.width = this.sourceWidth;
        buffCnvs.height = this.sourceHeight;
        context.drawImage(source, 0, 0, buffCnvs.width, buffCnvs.height);
        return this.filterPipeline.applyToCanvas(buffCnvs).then((bcnvs) => {
            this.contentCanvas.width = bcnvs.width;
            this.contentCanvas.height = bcnvs.height;
            const imgContext = this.contentCanvas.getContext("2d");
            imgContext.drawImage(bcnvs, 0, 0, bcnvs.width, bcnvs.height);
        }).then(() => {
            this.resize(this.editorContainerDiv.offsetWidth, this.editorContainerDiv.offsetHeight);
            this.handleZoomAfterContentUpdate();
        });
    }
    resize(containerWidth, containerHeight) {
        this.frameWidth = containerWidth;
        this.frameHeight = containerHeight;
        const imgRatio = this.contentCanvas.width / this.contentCanvas.height;
        const containerRatio = containerWidth / containerHeight;
        let hpadding = 0;
        let vpadding = 0;
        if (imgRatio > containerRatio) {
            vpadding = (containerHeight - containerWidth / imgRatio) / 2;
            this.editorDiv.style.height = `calc(100% - ${vpadding * 2}px)`;
            this.editorDiv.style.width = "";
        }
        else {
            hpadding = (containerWidth - containerHeight * imgRatio) / 2;
            this.editorDiv.style.height = "";
            this.editorDiv.style.width = `calc(100% - ${hpadding * 2}px)`;
        }
        this.editorDiv.style.padding = `${vpadding}px ${hpadding}px`;
        this.frameWidth = containerWidth - hpadding * 2;
        this.frameHeight = containerHeight - vpadding * 2;
        this.areaSelector.resize(this.frameWidth, this.frameHeight);
        this.regionsManager.resize(this.frameWidth, this.frameHeight);
        this.resetZoom();
    }
    resetWindow() {
        var cw = this.editorContainerDiv.clientWidth;
        var ch = this.editorContainerDiv.clientHeight;
        this.resize(cw, ch);
    }
    get RM() {
        return this.regionsManager;
    }
    get AS() {
        return this.areaSelector;
    }
    get FP() {
        return this.filterPipeline;
    }
    get ZM() {
        return this.zoomManager;
    }
    scaleRegionToSourceSize(regionData, sourceWidth, sourceHeight) {
        const sw = (sourceWidth !== undefined) ? sourceWidth : this.sourceWidth;
        const sh = (sourceHeight !== undefined) ? sourceHeight : this.sourceHeight;
        const xf = sw / this.frameWidth;
        const yf = sh / this.frameHeight;
        const rd = regionData.copy();
        rd.scale(xf, yf);
        return rd;
    }
    scaleRegionToFrameSize(regionData, sourceWidth, sourceHeight) {
        const sw = (sourceWidth !== undefined) ? sourceWidth : this.sourceWidth;
        const sh = (sourceHeight !== undefined) ? sourceHeight : this.sourceHeight;
        const xf = this.frameWidth / sw;
        const yf = this.frameHeight / sh;
        const rd = regionData.copy();
        rd.scale(xf, yf);
        return rd;
    }
    createSVGElement() {
        const svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
        svg.innerHTML = Editor.SVGDefsTemplate;
        svg.ondragstart = () => {
            return false;
        };
        svg.ondragend = () => {
            return false;
        };
        return svg;
    }
    createCanvasElement() {
        const canvas = document.createElement("canvas");
        return canvas;
    }
    createDivElement() {
        const div = document.createElement("div");
        return div;
    }
    onZoom(zoomType) {
        if (!this.zoomManager.isZoomEnabled) {
            throw new Error("Zoom feature is not enabled");
        }
        const zoomData = this.zoomManager.updateZoomScale(zoomType);
        if (zoomData) {
            var lastposx = this.editorContainerDiv.scrollLeft;
            var lastposy = this.editorContainerDiv.scrollTop;
            var cw = this.editorContainerDiv.clientWidth;
            var ch = this.editorContainerDiv.clientHeight;
            var sw = this.editorContainerDiv.scrollWidth;
            var sh = this.editorContainerDiv.scrollHeight;
            var xp = cw / sw;
            var yp = ch / sh;
            var mousex = lastposx + this.mousePoint.x;
            var mousey = lastposy + this.mousePoint.y;
            const newposx = (mousex / zoomData.previousZoomScale) * zoomData.currentZoomScale - this.mousePoint.x;
            const newposy = (mousey / zoomData.previousZoomScale) * zoomData.currentZoomScale - this.mousePoint.y;
            const scaledFrameWidth = (this.frameWidth / zoomData.previousZoomScale) * zoomData.currentZoomScale;
            const scaledFrameHeight = (this.frameHeight / zoomData.previousZoomScale) * zoomData.currentZoomScale;
            this.frameWidth = scaledFrameWidth;
            this.frameHeight = scaledFrameHeight;
            this.zoomEditorToScale(scaledFrameWidth, scaledFrameHeight);
            this.areaSelector.setlp(this.zoomManager.currentZoomScale, this.zoomManager.currentZoomScale, this.contentCanvas.width / this.editorContainerDiv.clientWidth, this.contentCanvas.height / this.editorContainerDiv.clientHeight);
            const imgRatio = this.contentCanvas.width / this.contentCanvas.height;
            this.areaSelector.resize(scaledFrameWidth, scaledFrameHeight);
            this.regionsManager.resize(scaledFrameWidth, scaledFrameHeight);
            this.editorContainerDiv.scrollTo(newposx, newposy);
            if (typeof this.onZoomEnd == "function") {
                this.onZoomEnd(zoomData);
            }
        }
    }
    resetZoom() {
        if (!this.zoomManager.isZoomEnabled) {
            throw new Error("Zoom feature is not enabled");
        }
        const zoomData = this.zoomManager.resetZoomScale();
        if (zoomData) {
            const scaledFrameWidth = this.frameWidth;
            const scaledFrameHeight = this.frameHeight;
            this.zoomEditorToScale(scaledFrameWidth, scaledFrameHeight);
            this.areaSelector.resize(scaledFrameWidth, scaledFrameHeight);
            this.areaSelector.setlp(this.zoomManager.currentZoomScale, this.zoomManager.currentZoomScale, this.contentCanvas.width / this.editorContainerDiv.clientWidth, this.contentCanvas.height / this.editorContainerDiv.clientHeight);
            this.regionsManager.resize(scaledFrameWidth, scaledFrameHeight);
            if (typeof this.onZoomEnd == "function") {
                this.onZoomEnd(zoomData);
            }
        }
    }
    tabZoom(fmin, fmax) {
        if (!this.zoomManager.isZoomEnabled) {
            throw new Error("Zoom feature is not enabled");
        }
        const scaledFrameWidth = (fmax.x - fmin.x) * 3;
        const scaledFrameHeight = (fmax.y - fmin.y) * 3;
        var sw = this.editorContainerDiv.scrollWidth;
        var sh = this.editorContainerDiv.scrollHeight;
        const zoomData = this.zoomManager.numberZoomScale(1);
        if (zoomData) {
            this.zoomEditorToScale(scaledFrameWidth, scaledFrameHeight);
            this.areaSelector.resize(scaledFrameWidth, scaledFrameHeight);
            this.areaSelector.setlp(this.zoomManager.currentZoomScale, this.zoomManager.currentZoomScale, this.contentCanvas.width / this.editorContainerDiv.clientWidth, this.contentCanvas.height / this.editorContainerDiv.clientHeight);
            this.regionsManager.resize(scaledFrameWidth, scaledFrameHeight);
            if (typeof this.onZoomEnd == "function") {
                this.onZoomEnd(zoomData);
            }
            this.editorContainerDiv.scrollTo(fmin.x, fmin.y);
        }
    }
    handleZoomAfterContentUpdate() {
        if (this.zoomManager.isZoomEnabled && !this.zoomManager.resetZoomOnContentLoad) {
            let zoomData = this.zoomManager.getZoomData();
            const scaledFrameWidth = this.frameWidth * zoomData.currentZoomScale;
            const scaledFrameHeight = this.frameHeight * zoomData.currentZoomScale;
            this.frameWidth = scaledFrameWidth;
            this.frameHeight = scaledFrameHeight;
            this.zoomEditorToScale(scaledFrameWidth, scaledFrameHeight);
            this.areaSelector.resize(scaledFrameWidth, scaledFrameHeight);
            this.regionsManager.resize(scaledFrameWidth, scaledFrameHeight);
            this.areaSelector.setlp(this.zoomManager.currentZoomScale, this.zoomManager.currentZoomScale, this.contentCanvas.width / this.editorContainerDiv.clientWidth, this.contentCanvas.height / this.editorContainerDiv.clientHeight);
        }
    }
    zoomEditorToScale(scaledFrameWidth, scaledFrameHeight) {
        if (!this.editorContainerDiv.offsetWidth) {
            this.editorContainerDiv = document.getElementsByClassName("CanvasToolsContainer")[0];
            this.editorDiv = document.getElementsByClassName("CanvasToolsEditor")[0];
        }
        const containerWidth = this.editorContainerDiv.offsetWidth;
        const containerHeight = this.editorContainerDiv.offsetHeight;
        let hpadding = 0;
        let vpadding = 0;
        if (scaledFrameWidth < containerWidth) {
            hpadding = (containerWidth - scaledFrameWidth) / 2;
            this.editorDiv.style.width = `calc(100% - ${hpadding * 2}px)`;
        }
        else {
            this.editorDiv.style.width = `${scaledFrameWidth}px`;
        }
        if (scaledFrameHeight < containerHeight) {
            vpadding = (containerHeight - scaledFrameHeight) / 2;
            this.editorDiv.style.height = `calc(100% - ${vpadding * 2}px)`;
        }
        else {
            this.editorDiv.style.height = `${scaledFrameHeight}px`;
        }
        if (hpadding && !vpadding) {
            hpadding = (this.editorContainerDiv.clientWidth - scaledFrameWidth) / 2;
            this.editorDiv.style.width = `calc(100% - ${hpadding * 2}px)`;
        }
        if (!hpadding && vpadding) {
            vpadding = (this.editorContainerDiv.clientHeight - scaledFrameHeight) / 2;
            this.editorDiv.style.height = `calc(100% - ${vpadding * 2}px)`;
        }
        this.editorDiv.style.padding = `${vpadding}px ${hpadding}px`;
        this.editorContainerDiv.focus();
    }
    subscribeToEvents() {
        window.addEventListener("resize", (e) => {
            if (this.autoResize) {
                this.resize(this.editorContainerDiv.offsetWidth, this.editorContainerDiv.offsetHeight);
            }
        });
        window.addEventListener("wheel", (e) => {
            e.preventDefault();
            if (e.altKey) {
                if (e.deltaY < 0) {
                    this.zoomManager.callbacks.onZoomingIn();
                }
                else {
                    this.zoomManager.callbacks.onZoomingOut();
                }
            }
        }, { passive: false });
        window.addEventListener("pointermove", (e) => {
            const rect = this.editorContainerDiv.getClientRects();
            if (this.read_only) {
                this.regionsManager.freeze();
            }
            if (rect) {
                this.mousePoint = new Point2D_1.Point2D(e.clientX - rect[0].left, e.clientY - rect[0].top);
                if (e.altKey && (e.buttons == 1 || e.buttons == 4)) {
                    const zoomData = this.zoomManager.getZoomData();
                    if (zoomData) {
                        var lastposx = this.editorContainerDiv.scrollLeft;
                        var lastposy = this.editorContainerDiv.scrollTop;
                        var mousex = lastposx - (this.mousePoint.x - this.lastmousePoint.x);
                        var mousey = lastposy - (this.mousePoint.y - this.lastmousePoint.y);
                        this.editorContainerDiv.scrollTo(mousex, mousey);
                    }
                }
                this.lastmousePoint = this.mousePoint;
            }
        });
        window.addEventListener("keydown", (e) => {
            if (e.keyCode == 32) {
                e.preventDefault();
            }
            if (e.keyCode == 9) {
                e.preventDefault();
            }
            if (e.code == "KeyZ") {
                this.resetWindow();
            }
            if (e.altKey && !this.isondraw) {
                this.regionsManager.freeze();
                this.isonmove = true;
            }
        });
        window.addEventListener("keyup", (e) => {
            if (e.key == "Alt" && !this.read_only && !this.isondraw) {
                this.regionsManager.unfreeze();
                this.isonmove = false;
            }
        });
    }
}
exports.Editor = Editor;
Editor.FullToolbarSet = [
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "none-select",
        iconFile: "none-selection.svg",
        tooltip: "Regions Manipulation (M)",
        key: ["M", "m"],
        actionCallback: (action, rm, sl) => {
            sl.setSelectionMode({ mode: ISelectorSettings_1.SelectionMode.NONE });
            sl.show();
        },
        activate: true,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SEPARATOR,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "point-select",
        iconFile: "point-selection.svg",
        tooltip: "Point-selection (P)",
        key: ["P", "p"],
        actionCallback: (action, rm, sl) => {
            sl.setSelectionMode({ mode: ISelectorSettings_1.SelectionMode.POINT });
            sl.show();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "rect-select",
        iconFile: "rect-selection.svg",
        tooltip: "Rectangular box (R)",
        key: ["R", "r"],
        actionCallback: (action, rm, sl) => {
            sl.setSelectionMode({ mode: ISelectorSettings_1.SelectionMode.RECT });
            sl.show();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "copy-select",
        iconFile: "copy-t-selection.svg",
        tooltip: "Template-based box (T)",
        key: ["T", "t"],
        actionCallback: (action, rm, sl) => {
            const regions = rm.getSelectedRegions();
            if (regions !== undefined && regions.length > 0) {
                const r = regions[0];
                sl.setSelectionMode({
                    mode: ISelectorSettings_1.SelectionMode.COPYRECT,
                    template: new Rect_1.Rect(r.regionData.width, r.regionData.height),
                });
            }
            else {
                sl.setSelectionMode({
                    mode: ISelectorSettings_1.SelectionMode.COPYRECT,
                    template: new Rect_1.Rect(40, 40),
                });
            }
            sl.show();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "polyline-select",
        iconFile: "polyline-selection.svg",
        tooltip: "Polyline-selection (Y)",
        key: ["Y", "y"],
        actionCallback: (action, rm, sl) => {
            sl.setSelectionMode({ mode: ISelectorSettings_1.SelectionMode.POLYLINE });
            sl.show();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "polygon-select",
        iconFile: "polygon-selection.svg",
        tooltip: "Polygon-selection (O)",
        key: ["O", "o"],
        actionCallback: (action, rm, sl) => {
            sl.setSelectionMode({ mode: ISelectorSettings_1.SelectionMode.POLYGON });
            sl.show();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SEPARATOR,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SEPARATOR,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SWITCH,
        action: "selection-lock",
        iconFile: "selection-lock.svg",
        tooltip: "Lock/unlock regions (L)",
        key: ["L", "l"],
        actionCallback: (action, rm, sl) => {
            rm.toggleFreezeMode();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SWITCH,
        action: "background-toggle",
        iconFile: "background-toggle.svg",
        tooltip: "Toggle Region Background (B)",
        key: ["B", "b"],
        actionCallback: (action, rm, sl) => {
            rm.toggleBackground();
        },
        activate: false,
    },
];
Editor.RectToolbarSet = [
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "none-select",
        iconFile: "none-selection.svg",
        tooltip: "Regions Manipulation (M)",
        key: ["M", "m"],
        actionCallback: (action, rm, sl) => {
            sl.setSelectionMode({ mode: ISelectorSettings_1.SelectionMode.NONE });
        },
        activate: true,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SEPARATOR,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "rect-select",
        iconFile: "rect-selection.svg",
        tooltip: "Rectangular box (R)",
        key: ["R", "r"],
        actionCallback: (action, rm, sl) => {
            sl.setSelectionMode({ mode: ISelectorSettings_1.SelectionMode.RECT });
            sl.show();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SELECTOR,
        action: "copy-select",
        iconFile: "copy-t-selection.svg",
        tooltip: "Template-based box (T)",
        key: ["T", "t"],
        actionCallback: (action, rm, sl) => {
            const regions = rm.getSelectedRegions();
            if (regions !== undefined && regions.length > 0) {
                const r = regions[0];
                sl.setSelectionMode({
                    mode: ISelectorSettings_1.SelectionMode.COPYRECT,
                    template: new Rect_1.Rect(r.regionData.width, r.regionData.height),
                });
            }
            else {
                sl.setSelectionMode({
                    mode: ISelectorSettings_1.SelectionMode.COPYRECT,
                    template: new Rect_1.Rect(40, 40),
                });
            }
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SEPARATOR,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.TRIGGER,
        action: "delete-all-select",
        iconFile: "delete-all-selection.svg",
        tooltip: "Delete all regions (D)",
        key: ["D", "d"],
        actionCallback: (action, rm, sl) => {
            rm.deleteAllRegions();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SEPARATOR,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SWITCH,
        action: "selection-lock",
        iconFile: "selection-lock.svg",
        tooltip: "Lock/unlock regions (L)",
        key: ["L", "l"],
        actionCallback: (action, rm, sl) => {
            rm.toggleFreezeMode();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.SWITCH,
        action: "background-toggle",
        iconFile: "background-toggle.svg",
        tooltip: "Toggle Region Background (B)",
        key: ["B", "b"],
        actionCallback: (action, rm, sl) => {
            rm.toggleBackground();
        },
        activate: false,
    }
];
Editor.ZoomIconGroupToolbar = [
    {
        type: ToolbarIcon_1.ToolbarItemType.TRIGGER,
        action: "zoom-in",
        iconFile: "zoom-in.svg",
        tooltip: "Zoom in (+)",
        key: ["+"],
        actionCallback: (action, rm, sl, zm) => {
            zm.callbacks.onZoomingIn();
        },
        activate: false,
    },
    {
        type: ToolbarIcon_1.ToolbarItemType.TRIGGER,
        action: "zoom-out",
        iconFile: "zoom-out.svg",
        tooltip: "Zoom out (-)",
        key: ["-"],
        actionCallback: (action, rm, sl, zm) => {
            zm.callbacks.onZoomingOut();
        },
        activate: false,
    }
];
Editor.SeparatorIconGroupToolbar = [
    {
        type: ToolbarIcon_1.ToolbarItemType.SEPARATOR,
    }
];
Editor.SVGDefsTemplate = `
        <defs>
            <filter id="black-glow">
                <feGaussianBlur in="SourceAlpha" stdDeviation="2" />
                <feOffset dx="0" dy="0" result="offsetblur" />
                <feComponentTransfer>
                    <feFuncA type="linear" slope="0.8" />
                </feComponentTransfer>
                <feMerge>
                    <feMergeNode />
                    <feMergeNode in="SourceGraphic" />
                </feMerge>
            </filter>
        </defs>`;
//# sourceMappingURL=CanvasTools.Editor.js.map