"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AreaSelector = void 0;
const Rect_1 = require("../Core/Rect");
const ISelectorSettings_1 = require("../Interface/ISelectorSettings");
const PointSelector_1 = require("./Selectors/PointSelector");
const PolylineSelector_1 = require("./Selectors/PolylineSelector");
const PolygonSelector_1 = require("./Selectors/PolygonSelector");
const RectCopySelector_1 = require("./Selectors/RectCopySelector");
const RectSelector_1 = require("./Selectors/RectSelector");
class AreaSelector {
    constructor(svgHost, callbacks) {
        this.isVisible = true;
        this.parentNode = svgHost;
        if (callbacks !== undefined) {
            this.callbacks = callbacks;
        }
        else {
            this.callbacks = {
                onLocked: null,
                onSelectionBegin: null,
                onSelectionEnd: null,
                onUnlocked: null,
            };
        }
        this.buildUIElements();
    }
    resize(width, height) {
        if (width !== undefined && height !== undefined) {
            this.boundRect.resize(width, height);
            this.verticalLine.node.setAttribute("x1", (width - 0.5).toString());
            this.verticalLine.node.setAttribute("x2", (width - 0.5).toString());
            this.verticalLine.node.setAttribute("y2", height.toString());
            this.horizontalLine.node.setAttribute("x1", "0");
            this.horizontalLine.node.setAttribute("x2", width.toString());
            this.horizontalLine.node.setAttribute("y1", (height - 0.5).toString());
            this.horizontalLine.node.setAttribute("y2", (height - 0.5).toString());
            this.vertical0Line.node.setAttribute("y2", height.toString());
            this.horizontal0Line.node.setAttribute("x2", width.toString());
        }
        else {
            this.boundRect.resize(this.parentNode.width.baseVal.value, this.parentNode.height.baseVal.value);
            this.verticalLine.node.setAttribute("x1", this.parentNode.width.baseVal.value.toString());
            this.verticalLine.node.setAttribute("x2", this.parentNode.width.baseVal.value.toString());
            this.verticalLine.node.setAttribute("y2", this.parentNode.height.baseVal.value.toString());
            this.horizontalLine.node.setAttribute("x1", "0");
            this.horizontalLine.node.setAttribute("x2", this.parentNode.width.baseVal.value.toString());
            this.horizontalLine.node.setAttribute("y1", this.parentNode.height.baseVal.value.toString());
            this.horizontalLine.node.setAttribute("y2", this.parentNode.height.baseVal.value.toString());
            this.vertical0Line.node.setAttribute("y2", this.parentNode.height.baseVal.value.toString());
            this.horizontal0Line.node.setAttribute("x2", this.parentNode.width.baseVal.value.toString());
        }
        if (this.selector !== null) {
            this.selector.resize(width, height);
        }
    }
    setlp(xp, yp, xp2, yp2) {
        if (this.selector) {
            this.selector.lxp = xp;
            this.selector.lyp = yp;
            this.selector.kxp = xp2;
            this.selector.kyp = yp2;
        }
    }
    enable() {
        if (this.selector !== null) {
            this.selector.enable();
            this.selector.resize(this.boundRect.width, this.boundRect.height);
        }
    }
    disable() {
        if (this.selector !== null) {
            this.selector.disable();
        }
    }
    show() {
        this.enable();
        this.isVisible = true;
    }
    hide() {
        this.disable();
        this.isVisible = false;
    }
    setSelectionMode(settings) {
        let selectionSettings;
        if (settings === null || settings === undefined) {
            selectionSettings = {
                mode: ISelectorSettings_1.SelectionMode.NONE,
            };
        }
        else if (settings.mode !== undefined) {
            selectionSettings = settings;
        }
        else {
            selectionSettings = { mode: settings };
        }
        if (this.selectorSettings === undefined || this.selectorSettings.mode !== selectionSettings.mode) {
            this.disable();
            this.selectorSettings = selectionSettings;
            if (this.selectorSettings.mode === ISelectorSettings_1.SelectionMode.NONE) {
                this.selector = null;
                return;
            }
            else if (this.selectorSettings.mode === ISelectorSettings_1.SelectionMode.COPYRECT) {
                this.selector = this.rectCopySelector;
                const template = this.selectorSettings.template;
                if (template !== undefined) {
                    this.rectCopySelector.setTemplate(template);
                }
                else {
                    this.rectCopySelector.setTemplate(AreaSelector.DefaultTemplateSize);
                }
            }
            else if (this.selectorSettings.mode === ISelectorSettings_1.SelectionMode.RECT) {
                this.selector = this.rectSelector;
            }
            else if (this.selectorSettings.mode === ISelectorSettings_1.SelectionMode.POINT) {
                this.selector = this.pointSelector;
            }
            else if (this.selectorSettings.mode === ISelectorSettings_1.SelectionMode.POLYLINE) {
                this.selector = this.polylineSelector;
            }
            else if (this.selectorSettings.mode === ISelectorSettings_1.SelectionMode.POLYGON) {
                this.selector = this.polygonSelector;
            }
            this.enable();
            if (this.isVisible) {
                this.show();
            }
            else {
                this.hide();
            }
        }
    }
    getSelectorSettings() {
        return this.selectorSettings;
    }
    buildUIElements() {
        this.paper = Snap(this.parentNode);
        this.boundRect = new Rect_1.Rect(this.parentNode.width.baseVal.value, this.parentNode.height.baseVal.value);
        this.areaSelectorLayer = this.paper.g();
        this.areaSelectorLayer.addClass("areaSelector");
        this.rectSelector = new RectSelector_1.RectSelector(this.parentNode, this.paper, this.boundRect, this.callbacks);
        this.rectCopySelector = new RectCopySelector_1.RectCopySelector(this.parentNode, this.paper, this.boundRect, new Rect_1.Rect(0, 0), this.callbacks);
        this.pointSelector = new PointSelector_1.PointSelector(this.parentNode, this.paper, this.boundRect, this.callbacks);
        this.polylineSelector = new PolylineSelector_1.PolylineSelector(this.parentNode, this.paper, this.boundRect, this.callbacks);
        this.polygonSelector = new PolygonSelector_1.PolygonSelector(this.parentNode, this.paper, this.boundRect, this.callbacks);
        this.selector = this.rectSelector;
        this.rectSelector.enable();
        this.rectCopySelector.disable();
        this.pointSelector.disable();
        this.polylineSelector.disable();
        this.polygonSelector.disable();
        this.selector.hide();
        this.areaSelectorLayer.add(this.rectSelector.node);
        this.areaSelectorLayer.add(this.rectCopySelector.node);
        this.areaSelectorLayer.add(this.pointSelector.node);
        this.areaSelectorLayer.add(this.polylineSelector.node);
        this.areaSelectorLayer.add(this.polygonSelector.node);
        this.verticalLine = this.paper.line(this.parentNode.width.baseVal.value, 0, this.parentNode.width.baseVal.value, this.parentNode.height.baseVal.value);
        this.horizontalLine = this.paper.line(0, this.parentNode.height.baseVal.value, this.parentNode.width.baseVal.value, this.parentNode.height.baseVal.value);
        this.vertical0Line = this.paper.line(0.5, 0, 0.5, this.parentNode.height.baseVal.value);
        this.horizontal0Line = this.paper.line(0, 0.5, this.parentNode.width.baseVal.value, 0.5);
        this.verticalLine.node.setAttribute("style", "stroke-width: 1;stroke: #6ff;");
        this.horizontalLine.node.setAttribute("style", "stroke-width: 1;stroke: #6ff;");
        this.vertical0Line.node.setAttribute("style", "stroke-width: 1;stroke: #6ff;");
        this.horizontal0Line.node.setAttribute("style", "stroke-width: 1;stroke: #6ff;");
        this.areaSelectorLayer.add(this.verticalLine);
        this.areaSelectorLayer.add(this.horizontalLine);
        this.areaSelectorLayer.add(this.vertical0Line);
        this.areaSelectorLayer.add(this.horizontal0Line);
    }
}
exports.AreaSelector = AreaSelector;
AreaSelector.DefaultTemplateSize = new Rect_1.Rect(20, 20);
//# sourceMappingURL=AreaSelector.js.map